import typer
from lumaCLI.models import Config
from rich import print
from pathlib import Path
from lumaCLI.utils import get_config, send_config, init_config
from lumaCLI.utils.common import DryRun, LumaURL, ConfigDir, Force

app = typer.Typer(
    name="config", no_args_is_help=True, pretty_exceptions_show_locals=False
)


@app.command(help="Initialize the configuration.")
def init(config_dir: Path = ConfigDir, force: bool = Force):
    try:
        init_config(config_dir=config_dir, force=force)
        print(f"[green]Config initialized at[/green] {config_dir}")
        raise typer.Exit(0)
    except FileExistsError:
        print("[red]Error![/red]")
        print(
            f"[red]Config files already exist at[/red] {config_dir}\n[yellow]If you want to override run with flag [/yellow][red]--force/-f[/red]"
        )
        raise typer.Exit(1)


@app.command(help="Display the current configuration information.")
def show(config_dir: Path = ConfigDir):
    try:
        config: Config = get_config(config_dir=config_dir)
    except FileNotFoundError:
        print("[red]Error![/red]")
        print(
            f"[red]Config files not found at[/red] {config_dir}\n[yellow]To generate config files use [/yellow][white]'luma config init'[/white]"
        )
        raise typer.Exit(1)

    print(config)
    raise typer.Exit(0)


@app.command(help="Send the current configuration information to luma")
def send(config_dir: Path = ConfigDir, luma_url: str = LumaURL, dry_run: bool = DryRun):
    # Retrieve the global config object
    try:
        config: Config = get_config(config_dir=config_dir)
    except FileNotFoundError:
        print("[red]Error![/red]")
        print(
            f"[red]Config files not found at[/red] {config_dir}\n[yellow]To generate config files use [/yellow][white]'luma config init'[/white]"
        )
        raise typer.Exit(1)

    # If in dry run mode, print the bundle and exit
    if dry_run:
        print(config.dict())
        raise typer.Exit(0)
    # Send the configuration and exit the program
    if config:
        response = send_config(config=config, luma_url=luma_url)
        if not response.ok:
            raise typer.Exit(1)

    else:
        print(
            f"[red]No Config detected under {config_dir}[/red]\n[yellow]To generate config files use [/yellow][white]'luma config init'[/white]"
        )



from lumaCLI import app
from typer.testing import CliRunner
from lumaCLI.tests.utils import METADATA_DIR


runner = CliRunner()


def test_ingest(test_server):
    result = runner.invoke(
        app,
        [
            "dbt",
            "ingest",
            "--metadata-dir",
            METADATA_DIR,
            "--luma-url",
            test_server,
        ],
    )

    assert result.exit_code == 0, result.output


def test_send_test_results(test_server):
    result = runner.invoke(
        app,
        [
            "dbt",
            "send-test-results",
            "--metadata-dir",
            METADATA_DIR,
            "--luma-url",
            test_server,
        ],
    )
    assert result.exit_code == 0, result.output



import json
import os
from pathlib import Path
from rich import print
from rich.panel import Panel


def validate_json(json_path: Path, endswith: str = ".json") -> bool:
    """
    Validates the provided file as a valid JSON file with the specified string suffix.

    Args:
        json_path (Path): Full path to the file to be validated.
        endswith (str, optional): Suffix that the file name should have. Defaults to ".json".

    Returns:
        bool: Returns True if validation is successful, False otherwise.
    """
    # Check if file exists
    if not json_path.is_file():
        error_message = (
            f"[red]Error[/red]: [yellow]{json_path.absolute()}[/yellow] "
            f"[blue]is not a file[/blue]"
        )
        print(Panel(error_message))
        return False

    # Check if file name ends with the specified suffix
    if not str(json_path).endswith(endswith):
        error_message = (
            f"[red]Error[/red]: [blue]File[/blue] "
            f"[yellow]{os.path.basename(json_path.absolute())}[/yellow] "
            f"[blue]does not have the required structure, it should end with "
            f"[/blue][yellow]'{endswith}'[/yellow]"
        )

        print(Panel(error_message))
        return False

    return True


def json_to_dict(json_path):
    """
    Converts the provided JSON file to a dictionary.

    Args:
        json_path (str): Full path to the JSON file.

    Returns:
        dict: Returns JSON data as a dictionary.
    """
    try:
        with open(json_path, "r") as json_file:
            # Load JSON data from file
            json_data: dict = json.load(json_file)
    except:
        json_data = None
    return json_data



from typing import Optional
import typer
from pathlib import Path
from rich import print
from rich.panel import Panel
from lumaCLI.models import Config, RequestInfo
from lumaCLI.models.dbt import ManifestDict
from lumaCLI.utils import (
    json_to_dict,
    validate_json,
    get_config,
    send_config,
    send_request_info,
)
from lumaCLI.utils.common import (
    ConfigDir,
    LumaURL,
    DryRun,
    NoConfig,
    MetadataDir,
)

# Create Typer application
app = typer.Typer(no_args_is_help=True, pretty_exceptions_show_locals=False)


@app.command()
def ingest(
    metadata_dir: Path = MetadataDir,
    luma_url: str = LumaURL,
    dry_run: bool = DryRun,
    config_dir: Path = ConfigDir,
    no_config: bool = NoConfig,
) -> RequestInfo:
    """
    Ingests a bundle of JSON files (manifest.json, catalog.json, sources.json, run_results.json) located in the specified directory to a Luma endpoint.
    If any of these files is not present in the directory, the command will fail. Uses the current working directory if 'metadata_dir' is not specified.
    """

    should_send_config = not no_config
    config = None
    # get_config

    if should_send_config:
        try:
            config: Config = get_config(config_dir=config_dir)
        except FileNotFoundError:
            print(
                Panel(
                    f"[blue]No config files found. Continuing with the operation...[/blue]"
                )
            )

    # Define JSON paths
    manifest_json_path = metadata_dir / "manifest.json"
    catalog_json_path = metadata_dir / "catalog.json"
    sources_json_path = metadata_dir / "sources.json"
    run_results_json_path = metadata_dir / "run_results.json"

    # Validate each JSON file
    

    

    # Ensure all JSON files are valid
    if not manifest_json_path.is_file():
        print(Panel(f"[red]{manifest_json_path.absolute()} is not a file[/red]"))
        raise typer.Exit(1)
    if not catalog_json_path.is_file():
        print(Panel(f"[red]{catalog_json_path.absolute()} is not a file[/red]"))
        raise typer.Exit(1)

    # Convert each JSON to dict
    manifest_dict: Optional[dict] = json_to_dict(json_path=manifest_json_path)
    catalog_dict: Optional[dict] = json_to_dict(json_path=catalog_json_path)
    sources_dict: Optional[dict] = json_to_dict(json_path=sources_json_path)
    run_results_dict: Optional[dict] = json_to_dict(json_path=run_results_json_path)


    # Validate manifest_dict using ManifestDict
    ManifestDict.validate(manifest_dict)


    # Define bundle dict
    bundle_dict = {
        "manifest_json": manifest_dict,
        "catalog_json": catalog_dict,
        "sources_json": sources_dict,
        "run_results_json": run_results_dict,
    }



    # If in dry run mode, print the bundle and exit
    if dry_run:
        print(bundle_dict)
        raise typer.Exit(0)

    endpoint = f"{luma_url}/api/v1/dbt/"

    # Create the request information
    request_info = RequestInfo(
        url=endpoint,
        method="POST",
        payload=bundle_dict,
        verify=False,
        timeout=(3.05, 60 * 30),
    )

    if config and should_send_config:
        config_response = send_config(config=config, luma_url=luma_url)

    response = send_request_info(request_info)
    if not response.ok:
        raise typer.Exit(1)

    return response


@app.command()
def send_test_results(
    metadata_dir: Path = MetadataDir,
    luma_url: str = LumaURL,
    dry_run: bool = DryRun,
    config_dir: Path = ConfigDir,
    no_config: bool = NoConfig,
) -> RequestInfo:
    """
    Sends the 'run_results.json' file located in the specified directory to a Luma endpoint.
    The command will fail if the 'run_results.json' file is not present in the directory. The current working directory is used if 'metadata_dir' is not specified.
    """

    should_send_config = not no_config
    config = None
    # get_config

    if should_send_config:
        try:
            config: Config = get_config(config_dir=config_dir)
        except FileNotFoundError:
            print(
                Panel(
                    f"[blue]No config files found. Continuing with the operation...[/blue]"
                )
            )

    # Define the path to 'run_results.json'
    run_results_path = Path(metadata_dir) / "run_results.json"

    # Validate 'run_results.json'
    is_run_results_json_valid = validate_json(run_results_path, "run_results.json")

    # Ensure 'run_results.json' is valid
    if not is_run_results_json_valid:
        print(f"[red]run_results.json not valid[/red]")
        raise typer.Exit(1)

    # Convert 'run_results.json' to dict
    run_results_dict = json_to_dict(json_path=run_results_path)

    # If in dry run mode, print the test results and exit
    if dry_run:
        print(run_results_dict)
        raise typer.Exit(0)

    endpoint = f"{luma_url}/api/v1/dbt/run_results/"

    # Create and return the request information for test results
    request_info = RequestInfo(
        url=endpoint,
        method="POST",
        payload=run_results_dict,
        verify=False,
        timeout=(3.05, 60 * 30),
    )

    if config and should_send_config:
        config_response = send_config(config=config, luma_url=luma_url)

    response = send_request_info(request_info)
    if not response.ok:
        raise typer.Exit(1)
    return response


# Run the application
if __name__ == "__main__":
    app()



import os

TESTS_DIR = os.path.dirname(__file__)
TEMP_DIR = os.path.join(TESTS_DIR, "temp")
METADATA_DIR = os.path.join(TESTS_DIR, "metadata_dir")
MANIFEST_JSON = os.path.join(METADATA_DIR, "manifest.json")
INVALID_JSON = os.path.join(METADATA_DIR, "invalid_json.json")
FILE_TXT = os.path.join(METADATA_DIR, "file.txt")
POSTGRES_DUMP_FILE = os.path.join(METADATA_DIR, "postgres_dump_file.sql")
LUMA_CONFIG_DIR = os.path.join(TESTS_DIR, ".luma")
INVALID_SCHEMA_LUMA_CONFIG_DIR = os.path.join(TESTS_DIR, ".invalid_schema_luma")



#!/usr/bin/env python

# Required imports
import typer
import urllib3

# Local module imports
import lumaCLI.commands.config as config_module
import lumaCLI.commands.dbt as dbt_module
import lumaCLI.commands.postgres as postgres_module

from lumaCLI.utils.common import CLI_NAME, __version__

# Disabling warnings related to insecure requests
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

# Create a Typer application with set properties
app = typer.Typer(
    name=CLI_NAME,  # Set the name of the CLI
    no_args_is_help=True,  # Show help message if no arguments are provided
    pretty_exceptions_show_locals=False,  # Do not display local variables when an exception occurs
    pretty_exceptions_enable=True,
    pretty_exceptions_short=True
)


# Version callback
def version_callback(show_version: bool):
    if show_version:
        typer.echo(__version__)
        raise typer.Exit()


# Add a --version option
@app.callback()
def main(
    version: bool = typer.Option(
        None,
        "--version",
        "-v",
        callback=version_callback,
        is_eager=True,
        help="Show the version and exit.",
    ),
):
    pass


# Add commands to the application
app.add_typer(dbt_module.app, name="dbt")
app.add_typer(postgres_module.app, name="postgres")
app.add_typer(config_module.app, name="config")


if __name__ == "__main__":
    app()
